@@ -769,6 +769,7 @@ double http_GetHdrQ(const struct http *hp, const char *hdr, const char *field);
 uint16_t http_GetStatus(const struct http *hp);
 const char *http_GetReq(const struct http *hp);
 int http_HdrIs(const struct http *hp, const char *hdr, const char *val);
+int http_IsHdr(const txt *hh, const char *hdr);
 uint16_t http_DissectRequest(struct sess *sp);
 uint16_t http_DissectResponse(struct worker *w, const struct http_conn *htc,
     struct http *sp);
@@ -894,7 +895,7 @@ void RES_StreamEnd(struct sess *sp);
 void RES_StreamPoll(const struct sess *sp);
 
 /* cache_vary.c */
-struct vsb *VRY_Create(const struct sess *sp, const struct http *hp);
+int VRY_Create(const struct sess *sp, const struct http *hp, struct vsb **psb);
 int VRY_Match(struct sess *sp, const uint8_t *vary);
 void VRY_Validate(const uint8_t *vary);
 
