@@ -251,7 +251,6 @@ vbe_Healthy(const struct vdi_simple *vs, const struct sess *sp)
 	unsigned i = 0, retval;
 	unsigned int threshold;
 	struct backend *backend;
-	uintptr_t target;
 	double now;
 
 	CHECK_OBJ_NOTNULL(sp, SESS_MAGIC);
@@ -259,26 +258,30 @@ vbe_Healthy(const struct vdi_simple *vs, const struct sess *sp)
 	backend = vs->backend;
 	CHECK_OBJ_NOTNULL(backend, BACKEND_MAGIC);
 
-	if (!backend->healthy)
+	if (backend->admin_health == ah_probe && !backend->healthy)
+		return (0);
+
+	if (backend->admin_health == ah_sick)
 		return (0);
 
 	/* VRT/VCC sets threshold to UINT_MAX to mark that it's not
 	 * specified by VCL (thus use param).
 	 */
-	if (vs->vrt->saintmode_threshold == UINT_MAX)
+	threshold = vs->vrt->saintmode_threshold;
+	if (threshold == UINT_MAX)
 		threshold = params->saintmode_threshold;
-	else
-		threshold = vs->vrt->saintmode_threshold;
 
-	/* Saintmode is disabled */
-	if (threshold == 0)
+	if (backend->admin_health == ah_healthy)
+		threshold = UINT_MAX;
+
+	/* Saintmode is disabled, or list is empty */
+	if (threshold == 0 || VTAILQ_EMPTY(&backend->troublelist))
 		return (1);
 
 	if (sp->objcore == NULL)
 		return (1);
 
 	now = sp->t_req;
-	target = (uintptr_t)(sp->objcore->objhead);
 
 	old = NULL;
 	retval = 1;
@@ -293,7 +296,7 @@ vbe_Healthy(const struct vdi_simple *vs, const struct sess *sp)
 			break;
 		}
 
-		if (tr->target == target) {
+		if (!memcmp(tr->digest, sp->digest, sizeof tr->digest)) {
 			retval = 0;
 			break;
 		}
