@@ -500,7 +500,7 @@ static struct cli_proto backend_cmds[] = {
 	{ "backend.list", "backend.list",
 	    "\tList all backends\n", 0, 1, "", cli_backend_list },
 	{ "backend.set_health", "backend.set_health matcher state",
-	    "\tShow a backend\n", 2, 2, "", cli_backend_set_health },
+	    "\tSet health status on a backend\n", 2, 2, "", cli_backend_set_health },
 	{ NULL }
 };
 
