@@ -82,6 +82,7 @@ struct vgz {
 	const char		*id;
 	struct ws		*tmp;
 	char			*tmp_snapshot;
+	int			last_i;
 
 	struct storage		*obuf;
 
@@ -205,8 +206,6 @@ VGZ_NewGzip(struct sess *sp, const char *id)
 	    16 + params->gzip_window,	/* Window bits (16=gzip + 15) */
 	    params->gzip_memlevel,	/* memLevel */
 	    Z_DEFAULT_STRATEGY);
-	if (i != Z_OK)
-		printf("deflateInit2() = %d\n", i);
 	assert(Z_OK == i);
 	return (vg);
 }
@@ -296,6 +295,7 @@ VGZ_Gunzip(struct vgz *vg, const void **pptr, size_t *plen)
 		if (vg->obuf != NULL)
 			vg->obuf->len += l;
 	}
+	vg->last_i = i;
 	if (i == Z_OK)
 		return (VGZ_OK);
 	if (i == Z_STREAM_END)
@@ -338,6 +338,7 @@ VGZ_Gzip(struct vgz *vg, const void **pptr, size_t *plen, enum vgz_flag flags)
 		if (vg->obuf != NULL)
 			vg->obuf->len += l;
 	}
+	vg->last_i = i;
 	if (i == Z_OK)
 		return (0);
 	if (i == Z_STREAM_END)
@@ -405,10 +406,11 @@ VGZ_UpdateObj(const struct vgz *vg, struct object *obj)
 
 /*--------------------------------------------------------------------*/
 
-void
+int
 VGZ_Destroy(struct vgz **vgp)
 {
 	struct vgz *vg;
+	int i;
 
 	vg = *vgp;
 	CHECK_OBJ_NOTNULL(vg, VGZ_MAGIC);
@@ -424,10 +426,19 @@ VGZ_Destroy(struct vgz **vgp)
 	if (vg->tmp != NULL)
 		WS_Reset(vg->tmp, vg->tmp_snapshot);
 	if (vg->dir == VGZ_GZ)
-		AZ(deflateEnd(&vg->vz));
+		i = deflateEnd(&vg->vz);
 	else
-		AZ(inflateEnd(&vg->vz));
+		i = inflateEnd(&vg->vz);
+	if (vg->last_i == Z_STREAM_END && i == Z_OK)
+		i = Z_STREAM_END;
 	FREE_OBJ(vg);
+	if (i == Z_OK)
+		return (VGZ_OK);
+	if (i == Z_STREAM_END)
+		return (VGZ_END);
+	if (i == Z_BUF_ERROR)
+		return (VGZ_STUCK);
+	return (VGZ_ERROR);
 }
 
 /*--------------------------------------------------------------------
@@ -462,28 +473,24 @@ vfp_gunzip_bytes(struct sess *sp, struct http_conn *htc, ssize_t bytes)
 			l = sizeof ibuf;
 			if (l > bytes)
 				l = bytes;
-			w = HTC_Read(htc, ibuf, l);
+			w = HTC_Read(sp->wrk, htc, ibuf, l);
 			if (w <= 0)
 				return (w);
 			VGZ_Ibuf(vg, ibuf, w);
 			bytes -= w;
 		}
 
-		if (VGZ_ObufStorage(sp, vg)) {
-			htc->error = "Could not get storage";
-			return (-1);
-		}
+		if (VGZ_ObufStorage(sp, vg))
+			return(-1);
 		i = VGZ_Gunzip(vg, &dp, &dl);
-		assert(i == VGZ_OK || i == VGZ_END);
+		if (i != VGZ_OK && i != VGZ_END)
+			return(FetchError(sp, "Gunzip data error"));
 		sp->obj->len += dl;
 		if (sp->wrk->do_stream)
 			RES_StreamPoll(sp);
 	}
-	if (i == Z_OK || i == Z_STREAM_END)
-		return (1);
-	htc->error = "See other message";
-	WSP(sp, SLT_FetchError, "Gunzip trouble (%d)", i);
-	return (-1);
+	assert(i == Z_OK || i == Z_STREAM_END);
+	return (1);
 }
 
 static int __match_proto__()
@@ -494,7 +501,12 @@ vfp_gunzip_end(struct sess *sp)
 	vg = sp->wrk->vgz_rx;
 	sp->wrk->vgz_rx = NULL;
 	CHECK_OBJ_NOTNULL(vg, VGZ_MAGIC);
-	VGZ_Destroy(&vg);
+	if (sp->wrk->fetch_failed) {
+		(void)VGZ_Destroy(&vg);
+		return(0);
+	}
+	if (VGZ_Destroy(&vg) != VGZ_END)
+		return(FetchError(sp, "Gunzip error at the very end"));
 	return (0);
 }
 
@@ -538,16 +550,14 @@ vfp_gzip_bytes(struct sess *sp, struct http_conn *htc, ssize_t bytes)
 			l = sizeof ibuf;
 			if (l > bytes)
 				l = bytes;
-			w = HTC_Read(htc, ibuf, l);
+			w = HTC_Read(sp->wrk, htc, ibuf, l);
 			if (w <= 0)
 				return (w);
 			VGZ_Ibuf(vg, ibuf, w);
 			bytes -= w;
 		}
-		if (VGZ_ObufStorage(sp, vg)) {
-			htc->error = "Could not get storage";
-			return (-1);
-		}
+		if (VGZ_ObufStorage(sp, vg))
+			return(-1);
 		i = VGZ_Gzip(vg, &dp, &dl, VGZ_NORMAL);
 		assert(i == Z_OK);
 		sp->obj->len += dl;
@@ -566,19 +576,25 @@ vfp_gzip_end(struct sess *sp)
 	int i;
 
 	vg = sp->wrk->vgz_rx;
-	sp->wrk->vgz_rx = NULL;
 	CHECK_OBJ_NOTNULL(vg, VGZ_MAGIC);
+	sp->wrk->vgz_rx = NULL;
+
+	if (sp->wrk->fetch_failed) {
+		(void)VGZ_Destroy(&vg);
+		return(0);
+	}
 	do {
 		VGZ_Ibuf(vg, "", 0);
 		if (VGZ_ObufStorage(sp, vg))
-			return (-1);
+			return(-1);
 		i = VGZ_Gzip(vg, &dp, &dl, VGZ_FINISH);
 		sp->obj->len += dl;
 	} while (i != Z_STREAM_END);
 	if (sp->wrk->do_stream)
 		RES_StreamPoll(sp);
 	VGZ_UpdateObj(vg, sp->obj);
-	VGZ_Destroy(&vg);
+	if (VGZ_Destroy(&vg) != VGZ_END)
+		return(FetchError(sp, "Gzip error at the very end"));
 	return (0);
 }
 
@@ -619,14 +635,12 @@ vfp_testgzip_bytes(struct sess *sp, struct http_conn *htc, ssize_t bytes)
 	AZ(vg->vz.avail_in);
 	while (bytes > 0) {
 		st = FetchStorage(sp, 0);
-		if (st == NULL) {
-			htc->error = "Could not get storage";
-			return (-1);
-		}
+		if (st == NULL)
+			return(-1);
 		l = st->space - st->len;
 		if (l > bytes)
 			l = bytes;
-		w = HTC_Read(htc, st->ptr + st->len, l);
+		w = HTC_Read(sp->wrk, htc, st->ptr + st->len, l);
 		if (w <= 0)
 			return (w);
 		bytes -= w;
@@ -639,23 +653,16 @@ vfp_testgzip_bytes(struct sess *sp, struct http_conn *htc, ssize_t bytes)
 		while (!VGZ_IbufEmpty(vg)) {
 			VGZ_Obuf(vg, obuf, sizeof obuf);
 			i = VGZ_Gunzip(vg, &dp, &dl);
-			if (i == VGZ_END && !VGZ_IbufEmpty(vg)) {
-				htc->error = "Junk after gzip data";
-				return (-1);
-			}
-			if (i != VGZ_OK && i != VGZ_END) {
-				htc->error = "See other message";
-				WSP(sp, SLT_FetchError,
-				    "Invalid Gzip data: %s", vg->vz.msg);
-				return (-1);
-			}
+			if (i == VGZ_END && !VGZ_IbufEmpty(vg))
+				return(FetchError(sp, "Junk after gzip data"));
+			if (i != VGZ_OK && i != VGZ_END)
+				return(FetchError2(sp,
+				   "Invalid Gzip data", vg->vz.msg));
+
 		}
 	}
-	if (i == VGZ_OK || i == VGZ_END)
-		return (1);
-	htc->error = "See other message";
-	WSP(sp, SLT_FetchError, "Gunzip trouble (%d)", i);
-	return (-1);
+	assert(i == VGZ_OK || i == VGZ_END);
+	return (1);
 }
 
 static int __match_proto__()
@@ -666,8 +673,13 @@ vfp_testgzip_end(struct sess *sp)
 	vg = sp->wrk->vgz_rx;
 	sp->wrk->vgz_rx = NULL;
 	CHECK_OBJ_NOTNULL(vg, VGZ_MAGIC);
+	if (sp->wrk->fetch_failed) {
+		(void)VGZ_Destroy(&vg);
+		return(0);
+	}
 	VGZ_UpdateObj(vg, sp->obj);
-	VGZ_Destroy(&vg);
+	if (VGZ_Destroy(&vg) != VGZ_END)
+		return(FetchError(sp, "TestGunzip error at the very end"));
 	return (0);
 }
 
