@@ -38,7 +38,6 @@
 #include <stdio.h>
 #include <string.h>
 #include <errno.h>
-#include <fcntl.h>
 #include <stdlib.h>
 #include <unistd.h>
 
@@ -206,13 +205,8 @@ vca_kqueue_main(void *arg)
 static void
 vca_kqueue_init(void)
 {
-	int i;
-
-	i = fcntl(vca_pipes[0], F_GETFL);
-	assert(i != -1);
-	i |= O_NONBLOCK;
-	i = fcntl(vca_pipes[0], F_SETFL, i);
-	assert(i != -1);
+	AZ(vnonblocking(vca_pipes[0]));
+	AZ(vnonblocking(vca_pipes[1]));
 
 	AZ(pthread_create(&vca_kqueue_thread, NULL, vca_kqueue_main, NULL));
 }
