@@ -1,6 +1,6 @@
 /*-
  * Copyright (c) 2006 Verdens Gang AS
- * Copyright (c) 2006-2009 Linpro AS
+ * Copyright (c) 2006-2011 Varnish Software AS
  * All rights reserved.
  *
  * Author: Poul-Henning Kamp <phk@phk.freebsd.dk>
@@ -26,29 +26,20 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: cli_common.h 4591 2010-02-26 12:12:00Z phk $
  */
 
 struct vlu;
-struct cls;
+struct VCLS;
 
 struct cli {
 	unsigned		magic;
 #define CLI_MAGIC		0x4038d570
 	struct vsb		*sb;
-	enum cli_status_e	result;
-	const char		*cmd;
+	enum VCLI_status_e	result;
+	char			*cmd;
 	unsigned		auth;
 	char			challenge[34];
 	char			*ident;
 	struct vlu		*vlu;
-	struct cls		*cls;
+	struct VCLS		*cls;
 };
-
-int cli_writeres(int fd, const struct cli *cli);
-int cli_readres(int fd, unsigned *status, char **ptr, double tmo);
-
-#define CLI_AUTH_RESPONSE_LEN		65	/* 64 hex + NUL */
-
-void CLI_response(int S_fd, const char *challenge,
-    char reponse[CLI_AUTH_RESPONSE_LEN]);
