@@ -1,6 +1,6 @@
 /*-
  * Copyright (c) 2006 Verdens Gang AS
- * Copyright (c) 2006-2009 Linpro AS
+ * Copyright (c) 2006-2010 Varnish Software AS
  * All rights reserved.
  *
  * Author: Poul-Henning Kamp <phk@phk.freebsd.dk>
@@ -26,8 +26,6 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: vbm.h 4475 2010-01-20 12:26:29Z phk $
- *
  * Self-sizeing bitmap operations
  */
 
@@ -75,6 +73,16 @@ vbit_init(unsigned initial)
 	return (vb);
 }
 
+static inline void
+vbit_destroy(struct vbitmap *vb)
+{
+
+	if (vb == NULL)
+		return;
+	free(vb->bits);
+	free(vb);
+}
+
 static inline void
 vbit_set(struct vbitmap *vb, unsigned bit)
 {
