@@ -295,7 +295,7 @@ iter_call(const struct vsc *vsc, VSC_iter_f *func, void *priv,
 		sp.class = t;						\
 		sp.ident = sha->ident;
 
-#define VSC_F(nn,tt,ll,ff,dd)						\
+#define VSC_F(nn,tt,ll,ff,dd,ee)					\
 		sp.name = #nn;						\
 		sp.fmt = #tt;						\
 		sp.flag = ff;						\
@@ -330,8 +330,8 @@ VSC_Iter(struct VSM_data *vd, VSC_iter_f *func, void *priv)
 		CHECK_OBJ_NOTNULL(sha, VSM_CHUNK_MAGIC);
 		if (strcmp(sha->class, VSC_CLASS))
 			continue;
-
-#define VSC_F(a,b,c,d,e)
+		/*lint -save -e525 -e539 */
+#define VSC_F(n,t,l,f,d,e)
 #define VSC_DONE(a,b,c)
 #define VSC_DO(U,l,t)						\
 		if (!strcmp(sha->type, t)) {			\
@@ -343,6 +343,7 @@ VSC_Iter(struct VSM_data *vd, VSC_iter_f *func, void *priv)
 #undef VSC_F
 #undef VSC_DO
 #undef VSC_DONE
+		/*lint -restore */
 		break;
 	}
 	return (i);
