@@ -671,8 +671,13 @@ vcc_CompileSource(const struct vcc *tl0, struct vsb *sb, struct source *sp)
 
 	/* Emit method functions */
 	for (i = 0; i < VCL_MET_MAX; i++) {
-		Fc(tl, 1, "\nstatic int\n");
-		Fc(tl, 1, "VGC_function_%s (struct sess *sp)\n",
+		Fh(tl, 1, "\nint\n");
+		Fh(tl, 1,
+		    "VGC_function_%s(struct sess *sp);\n",
+		    method_tab[i].name);
+		Fc(tl, 1, "\nint\n");
+		Fc(tl, 1,
+		    "VGC_function_%s(struct sess *sp)\n",
 		    method_tab[i].name);
 		AZ(VSB_finish(tl->fm[i]));
 		Fc(tl, 1, "{\n");
